@@ -87,7 +87,7 @@ function artifactInstalledFilePath(artifact) {
             throw "installSourceBase is not an absolute path";
         if (!artifact.filePath.startsWith(installSourceBase))
             throw "artifact file path doesn't start with the value of qbs.installSourceBase";
-        return FileInfo.joinPaths(targetDir, artifact.filePath.substr(installSourceBase.length + 1));
+        return FileInfo.joinPaths(targetDir, artifact.filePath.substr(installSourceBase.length));
     }
     return FileInfo.joinPaths(targetDir, artifact.fileName);
 }
@@ -331,7 +331,8 @@ var PropertyValidator = (function () {
             if (max !== undefined && value > max)
                 return false;
             return true;
-        }, "must be " + (!allowFloats ? "an integer " : "") + message.join(" and "));
+        }, "must be " + (!allowFloats ? "an integer " : "") + message.join(" and ")
+                + ", actual value: " + propertyValue);
     };
 
     PropertyValidator.prototype.addVersionValidator = function (propertyName, propertyValue, minComponents, maxComponents, allowSuffixes) {
@@ -347,7 +348,7 @@ var PropertyValidator = (function () {
         }, "must be a version number with " + (minComponents === maxComponents
                 ? minComponents : (minComponents + " to " + maxComponents))
                   + (minComponents === maxComponents && minComponents === 1
-                     ? " component" : " components"));
+                     ? " component" : " components") + ", actual value: " + propertyValue);
     };
 
     PropertyValidator.prototype.addFileNameValidator = function (propertyName, propertyValue) {
@@ -407,7 +408,7 @@ var PropertyValidator = (function () {
             errorMessage += "The following properties have invalid values:\n";
             lines = [];
             for (i in invalidProperties) {
-                for (j in invalidProperties[i]) {
+                for (j = 0; j < invalidProperties[i].length; ++j) {
                     lines.push(this.moduleName + "." + i + ": " + invalidProperties[i][j]);
                 }
             }
@@ -498,14 +499,14 @@ var BlackboxOutputArtifactTracker = (function () {
     return BlackboxOutputArtifactTracker;
 })();
 
-function guessArchitecture(m) {
-    function hasAnyOf(m, tokens) {
-        for (var i = 0; i < tokens.length; ++i) {
-            if (m[tokens[i]] !== undefined)
-                return true;
-        }
+function hasAnyOf(m, tokens) {
+    for (var i = 0; i < tokens.length; ++i) {
+        if (m[tokens[i]] !== undefined)
+            return true;
     }
+}
 
+function guessArchitecture(m) {
     var architecture;
     if (m) {
         // based on the search algorithm from qprocessordetection.h in qtbase
@@ -563,11 +564,86 @@ function guessArchitecture(m) {
                 architecture += "64";
         } else if (hasAnyOf(m, ["__AVR__"])) {
             architecture = "avr";
-            var avrSuffix = m["__AVR_ARCH__"];
-            if (avrSuffix)
-                architecture += avrSuffix;
+        } else if (hasAnyOf(m, ["__AVR32__"])) {
+            architecture = "avr32";
+        } else if (hasAnyOf(m, ["__MSP430__"])) {
+            architecture = "msp430";
+        } else if (hasAnyOf(m, ["__RL78__"])) {
+            architecture = "rl78";
+        } else if (hasAnyOf(m, ["__RX__"])) {
+            architecture = "rx";
+        } else if (hasAnyOf(m, ["__v850__"])) {
+            architecture = "v850";
+        } else if (hasAnyOf(m, ["__riscv"])) {
+            architecture = "riscv";
+        } else if (hasAnyOf(m, ["__xtensa__", "__XTENSA__"])) {
+            architecture = "xtensa";
+        } else if (hasAnyOf(m, ["__m68k__"])) {
+            architecture = "m68k";
+        } else if (hasAnyOf(m, ["__m32c__"])) {
+            architecture = "m32c";
+        } else if (hasAnyOf(m, ["__m32r__", "__M32R__"])) {
+            architecture = "m32r";
+        } else if (hasAnyOf(m, ["__sh__", "__SH__"])) {
+            architecture = "sh";
+        } else if (hasAnyOf(m, ["__CR16__"])) {
+            architecture = "cr16";
         }
     }
 
     return Utilities.canonicalArchitecture(architecture);
 }
+
+function guessTargetPlatform(m) {
+    if (m) {
+        if (hasAnyOf(m, ["__ANDROID__", "ANDROID"]))
+            return "android";
+        if (hasAnyOf(m, ["__QNXNTO__"]))
+            return "qnx";
+        if (hasAnyOf(m, ["__INTEGRITY"]))
+            return "integrity";
+        if (hasAnyOf(m, ["__vxworks"]))
+            return "vxworks";
+        if (hasAnyOf(m, ["__APPLE__"]))
+            return "darwin";
+        if (hasAnyOf(m, ["WIN32", "_WIN32", "__WIN32__", "__NT__"]))
+            return "windows";
+        if (hasAnyOf(m, ["_AIX"]))
+            return "aix";
+        if (hasAnyOf(m, ["hpux", "__hpux"]))
+            return "hpux";
+        if (hasAnyOf(m, ["__sun", "sun"]))
+            return "solaris";
+        if (hasAnyOf(m, ["__linux__", "__linux"]))
+            return "linux";
+        if (hasAnyOf(m, ["__FreeBSD__", "__DragonFly__", "__FreeBSD_kernel__"]))
+            return "freebsd";
+        if (hasAnyOf(m, ["__NetBSD__"]))
+            return "netbsd";
+        if (hasAnyOf(m, ["__OpenBSD__"]))
+            return "openbsd";
+        if (hasAnyOf(m, ["__GNU__"]))
+            return "hurd";
+        if (hasAnyOf(m, ["__HAIKU__"]))
+            return "haiku";
+    }
+}
+
+function toJSLiteral(v) {
+    if (v === undefined)
+        return "undefined";
+    return JSON.stringify(v);
+}
+
+function extractMacros(output) {
+    var m = {};
+    output.trim().split(/\r?\n/g).map(function (line) {
+        var prefix = "#define ";
+        if (!line.startsWith(prefix))
+            return;
+        var index = line.indexOf(" ", prefix.length);
+        if (index !== -1)
+            m[line.substr(prefix.length, index - prefix.length)] = line.substr(index + 1);
+    });
+    return m;
+}
