@@ -37,193 +37,242 @@ var Process = require("qbs.Process");
 var TemporaryDir = require("qbs.TemporaryDir");
 var TextFile = require("qbs.TextFile");
 
-function compilerName(qbs) {
-    switch (qbs.architecture) {
-    case "arm":
-        return "iccarm";
-    case "mcs51":
-        return "icc8051";
-    case "avr":
-        return "iccavr";
-    case "stm8":
-        return "iccstm8";
-    case "msp430":
-        return "icc430";
-    case "v850":
-        return "iccv850";
+function supportXLinker(architecture) {
+    return architecture === "78k" || architecture === "avr"
+        || architecture === "avr32" || architecture === "mcs51"
+        || architecture === "msp430" || architecture === "v850"
+        || architecture === "m68k" || architecture === "m32c"
+        || architecture === "r32c" || architecture === "m16c"
+        || architecture === "cr16";
+}
+
+function supportILinker(architecture) {
+    return architecture.startsWith("arm")
+        || architecture === "rh850" || architecture === "rl78"
+        || architecture === "rx" || architecture === "stm8"
+        || architecture === "sh" || architecture === "riscv";
+}
+
+function supportXArchiver(architecture) {
+    return architecture === "mcs51" || architecture === "avr"
+        || architecture === "msp430" || architecture === "v850"
+        || architecture === "78k" || architecture === "avr32"
+        || architecture === "m68k" || architecture === "m32c"
+        || architecture === "r32c" || architecture === "m16c"
+        || architecture === "cr16";
+}
+
+function supportIArchiver(architecture) {
+    return architecture.startsWith("arm")
+        || architecture === "stm8" || architecture === "rl78"
+        || architecture === "rx" || architecture === "rh850"
+        || architecture === "sh" || architecture === "riscv";
+}
+
+// It is a 'magic' IAR-specific target architecture code.
+function architectureCode(architecture) {
+    switch (architecture) {
     case "78k":
-        return "icc78k";
-    case "rl78":
-        return "iccrl78";
-    case "rx":
-        return "iccrx";
-    case "rh850":
-        return "iccrh850";
+        return "26";
+    case "avr":
+        return "90";
+    case "avr32":
+        return "82";
+    case "mcs51":
+        return "51";
+    case "msp430":
+        return "43";
+    case "v850":
+        return "85";
+    case "m68k":
+        return "68";
+    case "m32c":
+        return "48";
+    case "r32c":
+        return "53";
+    case "m16c":
+        return "34";
+    case "cr16":
+        return "45";
+    case "rh850": case "rl78": case "rx": case "stm8": case "sh": case "riscv":
+        return "";
+    default:
+        if (architecture.startsWith("arm"))
+            return "";
+        break;
     }
+}
+
+function compilerName(qbs) {
+    var architecture = qbs.architecture;
+    if (architecture.startsWith("arm"))
+        return  "iccarm";
+    else if (architecture === "mcs51")
+        return "icc8051";
+    else if (architecture === "avr")
+        return "iccavr";
+    else if (architecture === "stm8")
+        return "iccstm8";
+    else if (architecture === "msp430")
+        return "icc430";
+    else if (architecture === "v850")
+        return "iccv850";
+    else if (architecture === "78k")
+        return "icc78k";
+    else if (architecture === "rl78")
+        return "iccrl78";
+    else if (architecture === "rx")
+        return "iccrx";
+    else if (architecture === "rh850")
+        return "iccrh850";
+    else if (architecture === "avr32")
+        return "iccavr32";
+    else if (architecture === "sh")
+        return "iccsh";
+    else if (architecture === "riscv")
+        return "iccriscv";
+    else if (architecture === "m68k")
+        return "icccf";
+    else if (architecture === "m32c")
+        return "iccm32c";
+    else if (architecture === "r32c")
+        return "iccr32c";
+    else if (architecture === "m16c")
+        return "iccm16c";
+    else if (architecture === "cr16")
+        return "icccr16c";
     throw "Unable to deduce compiler name for unsupported architecture: '"
-            + qbs.architecture + "'";
+            + architecture + "'";
 }
 
 function assemblerName(qbs) {
-    switch (qbs.architecture) {
-    case "arm":
+    var architecture = qbs.architecture;
+    if (architecture.startsWith("arm"))
         return "iasmarm";
-    case "rl78":
+    else if (architecture === "rl78")
         return "iasmrl78";
-    case "rx":
+    else if (architecture === "rx")
         return "iasmrx";
-    case "rh850":
+    else if (architecture === "rh850")
         return "iasmrh850";
-    case "mcs51":
+    else if (architecture === "mcs51")
         return "a8051";
-    case "avr":
+    else if (architecture === "avr")
         return "aavr";
-    case "stm8":
+    else if (architecture === "stm8")
         return "iasmstm8";
-    case "msp430":
+    else if (architecture === "msp430")
         return "a430";
-    case "v850":
+    else if (architecture === "v850")
         return "av850";
-    case "78k":
+    else if (architecture === "78k")
         return "a78k";
-    }
+    else if (architecture === "avr32")
+        return "aavr32";
+    else if (architecture === "sh")
+        return "iasmsh";
+    else if (architecture === "riscv")
+        return "iasmriscv";
+    else if (architecture === "m68k")
+        return "acf";
+    else if (architecture === "m32c")
+        return "am32c";
+    else if (architecture === "r32c")
+        return "ar32c";
+    else if (architecture === "m16c")
+        return "am16c";
+    else if (architecture === "cr16")
+        return "acr16c";
     throw "Unable to deduce assembler name for unsupported architecture: '"
-            + qbs.architecture + "'";
+            + architecture + "'";
 }
 
 function linkerName(qbs) {
-    switch (qbs.architecture) {
-    case "arm":
-        return "ilinkarm";
-    case "stm8":
-        return "ilinkstm8";
-    case "rl78":
-        return "ilinkrl78";
-    case "rx":
-        return "ilinkrx";
-    case "rh850":
-        return "ilinkrh850";
-    case "mcs51":
-    case "avr":
-    case "msp430":
-    case "v850":
-    case "78k":
+    var architecture = qbs.architecture;
+    if (supportXLinker(architecture))
         return "xlink";
-    }
+    else if (supportILinker(architecture))
+        return architecture.startsWith("arm") ? "ilinkarm" : ("ilink" + architecture);
     throw "Unable to deduce linker name for unsupported architecture: '"
-            + qbs.architecture + "'";
+            + architecture + "'";
 }
 
 function archiverName(qbs) {
-    switch (qbs.architecture) {
-    case "arm":
-    case "stm8":
-    case "rl78":
-    case "rx":
-    case "rh850":
+    var architecture = qbs.architecture;
+    if (supportXArchiver(architecture))
+        return "xar";
+    else if (supportIArchiver(architecture))
         return "iarchive";
-    case "mcs51":
-    case "avr":
-    case "msp430":
-    case "v850":
-    case "78k":
-        return "xlib";
-    }
     throw "Unable to deduce archiver name for unsupported architecture: '"
-            + qbs.architecture + "'";
+            + architecture + "'";
 }
 
 function staticLibrarySuffix(qbs) {
-    switch (qbs.architecture) {
-    case "arm":
-    case "stm8":
-    case "rl78":
-    case "rx":
-    case "rh850":
-        return ".a";
-    case "mcs51":
-        return ".r51";
-    case "avr":
-        return ".r90";
-    case "msp430":
-        return ".r43";
-    case "v850":
-        return ".r85";
-    case "78k":
-        return ".r26";
+    var architecture = qbs.architecture;
+    var code = architectureCode(architecture);
+    if (code === undefined) {
+        throw "Unable to deduce static library suffix for unsupported architecture: '"
+                + architecture + "'";
     }
-    throw "Unable to deduce static library suffix for unsupported architecture: '"
-            + qbs.architecture + "'";
+    return (code !== "") ? (".r" + code) : ".a";
 }
 
 function executableSuffix(qbs) {
-    switch (qbs.architecture) {
-    case "arm":
-    case "stm8":
-    case "rl78":
-    case "rx":
-    case "rh850":
-        return ".out";
-    case "mcs51":
-        return qbs.debugInformation ? ".d51" : ".a51";
-    case "avr":
-        return qbs.debugInformation ? ".d90" : ".a90";
-    case "msp430":
-        return qbs.debugInformation ? ".d43" : ".a43";
-    case "v850":
-        return qbs.debugInformation ? ".d85" : ".a85";
-    case "78k":
-        return qbs.debugInformation ? ".d26" : ".a26";
+    var architecture = qbs.architecture;
+    var code = architectureCode(architecture);
+    if (code === undefined) {
+        throw "Unable to deduce executable suffix for unsupported architecture: '"
+                + architecture + "'";
     }
-    throw "Unable to deduce executable suffix for unsupported architecture: '"
-            + qbs.architecture + "'";
+    return (code !== "") ? ((qbs.debugInformation) ? (".d" + code) : (".a" + code)) : ".out";
 }
 
 function objectSuffix(qbs) {
-    switch (qbs.architecture) {
-    case "arm":
-    case "stm8":
-    case "rl78":
-    case "rx":
-    case "rh850":
-        return ".o";
-    case "mcs51":
-        return ".r51";
-    case "avr":
-        return ".r90";
-    case "msp430":
-        return ".r43";
-    case "v850":
-        return ".r85";
-    case "78k":
-        return ".r26";
+    var architecture = qbs.architecture;
+    var code = architectureCode(architecture);
+    if (code === undefined) {
+        throw "Unable to deduce object file suffix for unsupported architecture: '"
+                + architecture + "'";
     }
-    throw "Unable to deduce object file suffix for unsupported architecture: '"
-            + qbs.architecture + "'";
+    return (code !== "") ? (".r" + code) : ".o";
 }
 
 function imageFormat(qbs) {
-    switch (qbs.architecture) {
-    case "arm":
-    case "stm8":
-    case "rl78":
-    case "rx":
-    case "rh850":
-        return "elf";
-    case "mcs51":
-    case "avr":
-    case "msp430":
-    case "v850":
-    case "78k":
-        return "ubrof";
+    var architecture = qbs.architecture;
+    var code = architectureCode(architecture);
+    if (code === undefined) {
+        throw "Unable to deduce image format for unsupported architecture: '"
+                + architecture + "'";
     }
-    throw "Unable to deduce image format for unsupported architecture: '"
-            + qbs.architecture + "'";
+    return (code !== "") ? "ubrof" : "elf";
+}
+
+function guessArmArchitecture(core) {
+    var arch = "arm";
+    if (core === "__ARM4M__")
+        arch += "v4m";
+    else if (core === "__ARM4TM__")
+        arch += "v4tm";
+    else if (core === "__ARM5__")
+        arch += "v5";
+    else if (core === "__ARM5E__")
+        arch += "v5e";
+    else if (core === "__ARM6__")
+        arch += "v6";
+    else if (core === "__ARM6M__")
+        arch += "v6m";
+    else if (core === "__ARM6SM__")
+        arch += "v6sm";
+    else if (core === "__ARM7M__")
+        arch += "v7m";
+    else if (core === "__ARM7R__")
+        arch += "v7r";
+    return arch;
 }
 
 function guessArchitecture(macros) {
     if (macros["__ICCARM__"] === "1")
-        return "arm";
+        return guessArmArchitecture(macros["__CORE__"]);
     else if (macros["__ICC8051__"] === "1")
         return "mcs51";
     else if (macros["__ICCAVR__"] === "1")
@@ -242,6 +291,22 @@ function guessArchitecture(macros) {
         return "v850";
     else if (macros["__ICC78K__"] === "1")
         return "78k";
+    else if (macros["__ICCAVR32__"] === "1")
+        return "avr32";
+    else if (macros["__ICCSH__"] === "1")
+        return "sh";
+    else if (macros["__ICCRISCV__"] === "1")
+        return "riscv";
+    else if (macros["__ICCCF__"] === "1")
+        return "m68k";
+    else if (macros["__ICCM32C__"] === "1")
+        return "m32c";
+    else if (macros["__ICCR32C__"] === "1")
+        return "r32c";
+    else if (macros["__ICCM16C__"] === "1")
+        return "m16c";
+    else if (macros["__ICCCR16C__"] === "1")
+        return "cr16";
 }
 
 function guessEndianness(macros) {
@@ -253,25 +318,19 @@ function guessEndianness(macros) {
 function guessVersion(macros, architecture)
 {
     var version = parseInt(macros["__VER__"], 10);
-    switch (architecture) {
-    case "arm":
+    if (architecture.startsWith("arm")) {
         return { major: parseInt(version / 1000000),
             minor: parseInt(version / 1000) % 1000,
-            patch: parseInt(version) % 1000,
-            found: true }
-    case "mcs51":
-    case "avr":
-    case "stm8":
-    case "msp430":
-    case "rl78":
-    case "rx":
-    case "rh850":
-    case "v850":
-    case "78k":
+            patch: parseInt(version) % 1000 }
+    } else if (architecture === "mcs51" || architecture === "avr" || architecture === "stm8"
+               || architecture === "msp430" || architecture === "rl78" || architecture === "rx"
+               || architecture === "rh850" || architecture === "v850" || architecture === "78k"
+               || architecture === "avr32" || architecture === "sh" || architecture === "riscv"
+               || architecture === "m68k" || architecture === "m32c" || architecture === "r32c"
+               || architecture === "m16c" || architecture === "cr16") {
         return { major: parseInt(version / 100),
             minor: parseInt(version % 100),
-            patch: 0,
-            found: true }
+            patch: 0 }
     }
 }
 
@@ -282,6 +341,7 @@ function cppLanguageOption(compilerFilePath) {
     case "iccrl78":
     case "iccrx":
     case "iccrh850":
+    case "iccriscv":
         return "--c++";
     case "icc8051":
     case "iccavr":
@@ -289,6 +349,13 @@ function cppLanguageOption(compilerFilePath) {
     case "icc430":
     case "iccv850":
     case "icc78k":
+    case "iccavr32":
+    case "iccsh":
+    case "icccf":
+    case "iccm32c":
+    case "iccr32c":
+    case "iccm16c":
+    case "icccr16c":
         return "--ec++";
     }
     throw "Unable to deduce C++ language option for unsupported compiler: '"
@@ -308,12 +375,7 @@ function dumpMacros(compilerFilePath, tag) {
     var p = new Process();
     p.exec(compilerFilePath, args, true);
     var outFile = new TextFile(outFilePath, TextFile.ReadOnly);
-    var map = {};
-    outFile.readAll().trim().split(/\r?\n/g).map(function (line) {
-            var parts = line.split(" ", 3);
-            map[parts[1]] = parts[2];
-        });
-    return map;
+    return ModUtils.extractMacros(outFile.readAll());
 }
 
 function dumpDefaultPaths(compilerFilePath, tag) {
@@ -350,12 +412,9 @@ function dumpDefaultPaths(compilerFilePath, tag) {
         if (pass === 1)
             continue;
 
-        var parts = output.substring(startQuoteIndex + 1, endQuoteIndex).split("\n");
-        var includePath = "";
-        for (var i in parts)
-            includePath += parts[i].trim();
-
-        includePaths.push(includePath);
+        var path = output.substring(startQuoteIndex + 1, endQuoteIndex)
+            .replace(/[\s]{2,}/g, ' ');
+        includePaths.push(path);
     }
 
     return {
@@ -473,6 +532,10 @@ function compilerFlags(project, product, input, outputs, explicitlyDependsOn) {
     // Output.
     args.push("-o", outputs.obj[0].filePath);
 
+    var prefixHeaders = input.cpp.prefixHeaders;
+    for (var i in prefixHeaders)
+        args.push("--preinclude", prefixHeaders[i]);
+
     // Defines.
     var allDefines = [];
     var platformDefines = input.cpp.platformDefines;
@@ -491,9 +554,9 @@ function compilerFlags(project, product, input, outputs, explicitlyDependsOn) {
     var systemIncludePaths = input.cpp.systemIncludePaths;
     if (systemIncludePaths)
         allIncludePaths = allIncludePaths.uniqueConcat(systemIncludePaths);
-    var compilerIncludePaths = input.cpp.compilerIncludePaths;
-    if (compilerIncludePaths)
-        allIncludePaths = allIncludePaths.uniqueConcat(compilerIncludePaths);
+    var distributionIncludePaths = input.cpp.distributionIncludePaths;
+    if (distributionIncludePaths)
+        allIncludePaths = allIncludePaths.uniqueConcat(distributionIncludePaths);
     args = args.concat(allIncludePaths.map(function(include) { return "-I" + include }));
 
     // Silent output generation flag.
@@ -506,27 +569,32 @@ function compilerFlags(project, product, input, outputs, explicitlyDependsOn) {
     // Optimization flags.
     switch (input.cpp.optimization) {
     case "small":
-        args.push("-Ohs");
+        args.push("-Ohz");
         break;
     case "fast":
-        args.push("-Ohz");
+        args.push("-Ohs");
         break;
     case "none":
         args.push("-On");
         break;
     }
 
+    var architecture = input.qbs.architecture;
+
     // Warning level flags.
     switch (input.cpp.warningLevel) {
     case "none":
         args.push("--no_warnings");
         break;
     case "all":
-        if (input.qbs.architecture !== "78k") {
-            args.push("--deprecated_feature_warnings="
-                +"+attribute_syntax,"
-                +"+preprocessor_extensions,"
-                +"+segment_pragmas");
+        if (architecture !== "78k") {
+            if (architecture !== "avr32" && architecture !== "r32c"
+                && architecture !== "sh" && architecture !== "m16c") {
+                args.push("--deprecated_feature_warnings="
+                    +"+attribute_syntax,"
+                    +"+preprocessor_extensions,"
+                    +"+segment_pragmas");
+            }
             if (tag === "cpp")
                 args.push("--warn_about_c_style_casts");
         }
@@ -536,7 +604,7 @@ function compilerFlags(project, product, input, outputs, explicitlyDependsOn) {
         args.push("--warnings_are_errors");
 
     // C language version flags.
-    if (tag === "c" && (input.qbs.architecture !== "78k")) {
+    if (tag === "c" && (architecture !== "78k")) {
         var knownValues = ["c89"];
         var cLanguageVersion = Cpp.languageVersion(
                     input.cpp.cLanguageVersion, knownValues, "C");
@@ -551,19 +619,11 @@ function compilerFlags(project, product, input, outputs, explicitlyDependsOn) {
         }
     }
 
-    // Architecture specific flags.
-    switch (input.qbs.architecture) {
-    case "arm":
-    case "rl78":
-    case "rx":
-    case "rh850":
-        // Byte order flags.
-        var endianness = input.cpp.endianness;
-        if (endianness && (input.qbs.architecture === "arm"
-                || input.qbs.architecture === "rx")) {
-            args.push("--endian=" + endianness);
-        }
-        if (tag === "cpp") {
+    // C++ language version flags.
+    if (tag === "cpp") {
+        if (architecture.startsWith("arm")
+                || architecture === "rl78" || architecture === "rx"
+                || architecture ===  "rh850" || architecture === "riscv") {
             // Enable C++ language flags.
             args.push("--c++");
             // Exceptions flags.
@@ -572,22 +632,26 @@ function compilerFlags(project, product, input, outputs, explicitlyDependsOn) {
             // RTTI flags.
             if (!input.cpp.enableRtti)
                 args.push("--no_rtti");
-        }
-        break;
-    case "stm8":
-    case "mcs51":
-    case "avr":
-    case "msp430":
-    case "v850":
-    case "78k":
-        // Enable C++ language flags.
-        if (tag === "cpp")
+        } else if (architecture === "stm8" || architecture === "mcs51"
+                   || architecture === "avr" || architecture === "msp430"
+                   || architecture === "v850" || architecture === "78k"
+                   || architecture === "avr32" || architecture === "sh"
+                   || architecture === "m68k" || architecture === "m32c"
+                   || architecture === "r32c" || architecture === "m16c"
+                   || architecture === "cr16") {
             args.push("--ec++");
-        break;
+        }
+    }
+
+    // Byte order flags.
+    if (architecture.startsWith("arm") || architecture === "rx") {
+        var endianness = input.cpp.endianness;
+        if (endianness)
+            args.push("--endian=" + endianness);
     }
 
     // Listing files generation flag.
-    if (product.cpp.generateCompilerListingFiles)
+    if (input.cpp.generateCompilerListingFiles)
         args.push("-l", outputs.lst[0].filePath);
 
     // Misc flags.
@@ -616,9 +680,9 @@ function assemblerFlags(project, product, input, outputs, explicitlyDependsOn) {
     var systemIncludePaths = input.cpp.systemIncludePaths;
     if (systemIncludePaths)
         allIncludePaths = allIncludePaths.uniqueConcat(systemIncludePaths);
-    var compilerIncludePaths = input.cpp.compilerIncludePaths;
-    if (compilerIncludePaths)
-        allIncludePaths = allIncludePaths.uniqueConcat(compilerIncludePaths);
+    var distributionIncludePaths = input.cpp.distributionIncludePaths;
+    if (distributionIncludePaths)
+        allIncludePaths = allIncludePaths.uniqueConcat(distributionIncludePaths);
     args = args.concat(allIncludePaths.map(function(include) { return "-I" + include }));
 
     // Debug information flags.
@@ -626,11 +690,12 @@ function assemblerFlags(project, product, input, outputs, explicitlyDependsOn) {
         args.push("-r");
 
     // Architecture specific flags.
-    switch (input.qbs.architecture) {
-    case "stm8":
-    case "rl78":
-    case "rx":
-    case "rh850":
+    var architecture = input.qbs.architecture;
+    if (architecture === "stm8" || architecture === "rl78"
+            || architecture === "rx" || architecture === "rh850"
+            || architecture === "avr32" || architecture === "sh"
+            || architecture === "riscv" || architecture === "m68k"
+            || architecture === "r32c" || architecture === "cr16") {
         // Silent output generation flag.
         args.push("--silent");
         // Warning level flags.
@@ -638,27 +703,24 @@ function assemblerFlags(project, product, input, outputs, explicitlyDependsOn) {
             args.push("--no_warnings");
         if (input.cpp.treatWarningsAsErrors)
             args.push("--warnings_are_errors");
-        break;
-    default:
+    } else {
         // Silent output generation flag.
         args.push("-S");
         // Warning level flags.
         args.push("-w" + (input.cpp.warningLevel === "none" ? "-" : "+"));
-        break;
     }
 
     // Listing files generation flag.
-    if (product.cpp.generateAssemblerListingFiles)
+    if (input.cpp.generateAssemblerListingFiles)
         args.push("-l", outputs.lst[0].filePath);
 
     // Misc flags.
     args = args.concat(ModUtils.moduleProperty(input, "platformFlags", tag),
-                       ModUtils.moduleProperty(input, "flags", tag),
-                       ModUtils.moduleProperty(input, "driverFlags", tag));
+                       ModUtils.moduleProperty(input, "flags", tag));
     return args;
 }
 
-function linkerFlags(project, product, input, outputs) {
+function linkerFlags(project, product, inputs, outputs) {
     var args = [];
 
     // Inputs.
@@ -676,7 +738,6 @@ function linkerFlags(project, product, input, outputs) {
     var distributionLibraryPaths = product.cpp.distributionLibraryPaths;
     if (distributionLibraryPaths)
         allLibraryPaths = allLibraryPaths.uniqueConcat(distributionLibraryPaths);
-    args = args.concat(allLibraryPaths.map(function(path) { return '-L' + path }));
 
     // Library dependencies.
     var libraryDependencies = collectLibraryDependencies(product);
@@ -688,12 +749,9 @@ function linkerFlags(project, product, input, outputs) {
         ? inputs.linkerscript.map(function(a) { return a.filePath; }) : [];
 
     // Architecture specific flags.
-    switch (product.qbs.architecture) {
-    case "arm":
-    case "stm8":
-    case "rl78":
-    case "rx":
-    case "rh850":
+    var architecture = product.qbs.architecture;
+    if (supportILinker(architecture)) {
+        args = args.concat(allLibraryPaths.map(function(path) { return '-L' + path }));
         // Silent output generation flag.
         args.push("--silent");
         // Map file generation flag.
@@ -704,18 +762,14 @@ function linkerFlags(project, product, input, outputs) {
             args.push("--entry", product.cpp.entryPoint);
         // Linker scripts flags.
         linkerScripts.forEach(function(script) { args.push("--config", script); });
-        break;
-    case "mcs51":
-    case "avr":
-    case "msp430":
-    case "v850":
-    case "78k":
+    } else if (supportXLinker(architecture)) {
+        args = args.concat(allLibraryPaths.map(function(path) { return '-I' + path }));
         // Silent output generation flag.
         args.push("-S");
         // Debug information flag.
         if (product.cpp.debugInformation)
             args.push("-rt");
-         // Map file generation flag.
+        // Map file generation flag.
         if (product.cpp.generateLinkerMapFile)
             args.push("-l", outputs.mem_map[0].filePath);
         // Entry point flag.
@@ -723,7 +777,6 @@ function linkerFlags(project, product, input, outputs) {
             args.push("-s", product.cpp.entryPoint);
         // Linker scripts flags.
         linkerScripts.forEach(function(script) { args.push("-f", script); });
-        break;
     }
 
     // Misc flags.
@@ -731,7 +784,7 @@ function linkerFlags(project, product, input, outputs) {
     return args;
 }
 
-function archiverFlags(project, product, input, outputs) {
+function archiverFlags(project, product, inputs, outputs) {
     var args = [];
 
     // Inputs.
@@ -739,7 +792,9 @@ function archiverFlags(project, product, input, outputs) {
         args = args.concat(inputs.obj.map(function(obj) { return obj.filePath }));
 
     // Output.
-    args.push("--create");
+    var architecture = product.qbs.architecture;
+    if (supportIArchiver(architecture))
+        args.push("--create");
     args.push("-o", outputs.staticlibrary[0].filePath);
 
     return args;
@@ -765,7 +820,7 @@ function prepareAssembler(project, product, inputs, outputs, input, output, expl
 
 function prepareLinker(project, product, inputs, outputs, input, output) {
     var primaryOutput = outputs.application[0];
-    var args = linkerFlags(project, product, input, outputs);
+    var args = linkerFlags(project, product, inputs, outputs);
     var linkerPath = product.cpp.linkerPath;
     var cmd = new Command(linkerPath, args);
     cmd.description = "linking " + primaryOutput.fileName;
@@ -774,7 +829,7 @@ function prepareLinker(project, product, inputs, outputs, input, output) {
 }
 
 function prepareArchiver(project, product, inputs, outputs, input, output) {
-    var args = archiverFlags(project, product, input, outputs);
+    var args = archiverFlags(project, product, inputs, outputs);
     var archiverPath = product.cpp.archiverPath;
     var cmd = new Command(archiverPath, args);
     cmd.description = "linking " + output.fileName;
