@@ -28,12 +28,29 @@
 **
 ****************************************************************************/
 
+var Cpp = require("cpp.js");
 var File = require("qbs.File");
 var FileInfo = require("qbs.FileInfo");
 var ModUtils = require("qbs.ModUtils");
 var Utilities = require("qbs.Utilities");
 var WindowsUtils = require("qbs.WindowsUtils");
 
+function effectiveLinkerPath(product, inputs) {
+    if (product.cpp.linkerMode === "automatic") {
+        var compiler = product.cpp.compilerPath;
+        if (compiler) {
+            if (inputs.obj || inputs.staticlibrary) {
+                console.log("Found C/C++ objects, using compiler as a linker for " + product.name);
+                return compiler;
+            }
+        }
+
+        console.log("Found no C-language objects, choosing system linker for " + product.name);
+    }
+
+    return product.cpp.linkerPath;
+}
+
 function handleCpuFeatures(input, flags) {
     if (!input.qbs.architecture)
         return;
@@ -57,25 +74,57 @@ function handleCpuFeatures(input, flags) {
     }
 }
 
+function hasCxx17Option(input)
+{
+    // Probably this is not the earliest version to support the flag, but we have tested this one
+    // and it's a pain to find out the exact minimum.
+    return Utilities.versionCompare(input.cpp.compilerVersion, "19.12.25831") >= 0
+            || (input.qbs.toolchain.contains("clang-cl") && input.cpp.compilerVersionMajor >= 7);
+}
+
+function supportsExternalIncludesOption(input) {
+    if (input.qbs.toolchain.contains("clang-cl"))
+        return false; // Exclude clang-cl.
+    // This option was introcuded since MSVC 2017 v15.6 (aka _MSC_VER 19.13).
+    // But due to some MSVC bugs:
+    // * https://developercommunity.visualstudio.com/content/problem/181006/externali-include-paths-not-working.html
+    // this option has been fixed since MSVC 2017 update 9, v15.9 (aka _MSC_VER 19.16).
+    return Utilities.versionCompare(input.cpp.compilerVersion, "19.16") >= 0;
+}
+
 function addLanguageVersionFlag(input, args) {
-    var cxxVersion = input.cpp.cxxLanguageVersion;
+    var cxxVersion = Cpp.languageVersion(input.cpp.cxxLanguageVersion,
+                                         ["c++17", "c++14", "c++11", "c++98"], "C++");
     if (!cxxVersion)
         return;
 
     // Visual C++ 2013, Update 3
-    var hasStdOption = Utilities.versionCompare(input.cpp.compilerVersion, "18.00.30723") >= 0;
+    var hasStdOption = Utilities.versionCompare(input.cpp.compilerVersion, "18.00.30723") >= 0
+            // or clang-cl
+            || input.qbs.toolchain.contains("clang-cl");
     if (!hasStdOption)
         return;
 
     var flag;
     if (cxxVersion === "c++14")
         flag = "/std:c++14";
+    else if (cxxVersion === "c++17" && hasCxx17Option(input))
+        flag = "/std:c++17";
     else if (cxxVersion !== "c++11" && cxxVersion !== "c++98")
         flag = "/std:c++latest";
     if (flag)
         args.push(flag);
 }
 
+function handleClangClArchitectureFlags(product, architecture, flags) {
+    if (product.qbs.toolchain.contains("clang-cl")) {
+        if (architecture === "x86")
+            flags.push("-m32");
+        else if (architecture === "x86_64")
+            flags.push("-m64");
+    }
+}
+
 function prepareCompiler(project, product, inputs, outputs, input, output, explicitlyDependsOn) {
     var i;
     var debugInformation = input.cpp.debugInformation;
@@ -121,6 +170,8 @@ function prepareCompiler(project, product, inputs, outputs, input, output, expli
         break;
     }
 
+    handleClangClArchitectureFlags(product, input.cpp.architecture, args);
+
     if (debugInformation) {
         if (product.cpp.separateDebugInformation)
             args.push('/Zi');
@@ -136,6 +187,10 @@ function prepareCompiler(project, product, inputs, outputs, input, output, expli
         args.push(rtl);
     }
 
+    var driverFlags = product.cpp.driverFlags;
+    if (driverFlags)
+        args = args.concat(driverFlags);
+
     // warnings:
     var warningLevel = input.cpp.warningLevel;
     if (warningLevel === 'none')
@@ -144,15 +199,28 @@ function prepareCompiler(project, product, inputs, outputs, input, output, expli
         args.push('/Wall')
     if (input.cpp.treatWarningsAsErrors)
         args.push('/WX')
-    var allIncludePaths = [];
     var includePaths = input.cpp.includePaths;
-    if (includePaths)
-        allIncludePaths = allIncludePaths.uniqueConcat(includePaths);
+    if (includePaths) {
+        args = args.concat([].uniqueConcat(includePaths).map(function(path) {
+            return '/I' + FileInfo.toWindowsSeparators(path);
+        }));
+    }
+
+    var allSystemIncludePaths = [];
     var systemIncludePaths = input.cpp.systemIncludePaths;
     if (systemIncludePaths)
-        allIncludePaths = allIncludePaths.uniqueConcat(systemIncludePaths);
-    for (i in allIncludePaths)
-        args.push('/I' + FileInfo.toWindowsSeparators(allIncludePaths[i]))
+        allSystemIncludePaths = allSystemIncludePaths.uniqueConcat(systemIncludePaths);
+    var distributionIncludePaths = input.cpp.distributionIncludePaths;
+    if (distributionIncludePaths)
+        allSystemIncludePaths = allSystemIncludePaths.uniqueConcat(distributionIncludePaths);
+    var includeFlag = "/I";
+    if (supportsExternalIncludesOption(input)) {
+        args.push("/experimental:external");
+        includeFlag = "/external:I"
+    }
+    allSystemIncludePaths.forEach(function(path) {
+        args.push(includeFlag, FileInfo.toWindowsSeparators(path)); });
+
     var allDefines = [];
     var platformDefines = input.cpp.platformDefines;
     if (platformDefines)
@@ -174,7 +242,14 @@ function prepareCompiler(project, product, inputs, outputs, input, output, expli
         }
     }
 
-    var objOutput = outputs.obj ? outputs.obj[0] : undefined
+    if (product.cpp.debugInformation && product.cpp.separateDebugInformation)
+        args.push("/Fd" + product.targetName + ".cl" + product.cpp.debugInfoSuffix);
+
+    if (input.cpp.generateCompilerListingFiles)
+        args.push("/Fa" + FileInfo.toWindowsSeparators(outputs.lst[0].filePath));
+
+    var objectMap = outputs.obj || outputs.intermediate_obj
+    var objOutput = objectMap ? objectMap[0] : undefined
     args.push('/Fo' + FileInfo.toWindowsSeparators(objOutput.filePath))
     args.push(FileInfo.toWindowsSeparators(input.filePath))
 
@@ -195,10 +270,20 @@ function prepareCompiler(project, product, inputs, outputs, input, output, expli
     var pchInputs = explicitlyDependsOn[tag + "_pch"];
     if (pchOutput) {
         // create PCH
-        args.push("/Yc");
-        args.push("/Fp" + FileInfo.toWindowsSeparators(pchOutput.filePath));
-        args.push("/Fo" + FileInfo.toWindowsSeparators(objOutput.filePath));
-        args.push(FileInfo.toWindowsSeparators(input.filePath));
+        if (input.qbs.toolchain.contains("clang-cl")) {
+            // clang-cl does not support /Yc flag without filename
+            args.push("/Yc" + FileInfo.toWindowsSeparators(input.filePath));
+            // clang-cl complains when pch file is not included
+            args.push("/FI" + FileInfo.toWindowsSeparators(input.filePath));
+            args.push("/Fp" + FileInfo.toWindowsSeparators(pchOutput.filePath));
+            args.push("/Fo" + FileInfo.toWindowsSeparators(objOutput.filePath));
+        } else { // real msvc
+            args.push("/Yc");
+            args.push("/Fp" + FileInfo.toWindowsSeparators(pchOutput.filePath));
+            args.push("/Fo" + FileInfo.toWindowsSeparators(objOutput.filePath));
+            args.push(FileInfo.toWindowsSeparators(input.filePath));
+        }
+
     } else if (pchInputs && pchInputs.length === 1
                && ModUtils.moduleProperty(input, "usePrecompiledHeader", tag)) {
         // use PCH
@@ -232,6 +317,7 @@ function prepareCompiler(project, product, inputs, outputs, input, output, expli
     if (pchOutput)
         cmd.description += ' (' + tag + ')';
     cmd.highlight = "compiler";
+    cmd.jobPool = "compiler";
     cmd.workingDirectory = product.buildDirectory;
     cmd.responseFileUsagePrefix = '@';
     // cl.exe outputs the cpp file name. We filter that out.
@@ -247,8 +333,8 @@ function collectLibraryDependencies(product) {
     var seen = {};
     var result = [];
 
-    function addFilePath(filePath, wholeArchive) {
-        result.push({ filePath: filePath, wholeArchive: wholeArchive });
+    function addFilePath(filePath, wholeArchive, productName) {
+        result.push({ filePath: filePath, wholeArchive: wholeArchive, productName: productName });
     }
 
     function addArtifactFilePaths(dep, artifacts) {
@@ -256,11 +342,17 @@ function collectLibraryDependencies(product) {
             return;
         var artifactFilePaths = artifacts.map(function(a) { return a.filePath; });
         var wholeArchive = dep.parameters.cpp && dep.parameters.cpp.linkWholeArchive;
-        for (var i = 0; i < artifactFilePaths.length; ++i)
-            addFilePath(artifactFilePaths[i], wholeArchive);
+        var artifactsAreImportLibs = artifacts.length > 0
+                && artifacts[0].fileTags.contains("dynamiclibrary_import");
+        for (var i = 0; i < artifactFilePaths.length; ++i) {
+            addFilePath(artifactFilePaths[i], wholeArchive,
+                        artifactsAreImportLibs ? dep.name : undefined);
+        }
     }
 
     function addExternalLibs(obj) {
+        if (!obj.cpp)
+            return;
         function ensureArray(a) {
             return Array.isArray(a) ? a : [];
         }
@@ -280,7 +372,6 @@ function collectLibraryDependencies(product) {
     function traverse(dep) {
         if (seen.hasOwnProperty(dep.name))
             return;
-        seen[dep.name] = true;
 
         if (dep.parameters.cpp && dep.parameters.cpp.link === false)
             return;
@@ -289,10 +380,12 @@ function collectLibraryDependencies(product) {
         var dynamicLibraryArtifacts = staticLibraryArtifacts
                 ? null : dep.artifacts["dynamiclibrary_import"];
         if (staticLibraryArtifacts) {
+            seen[dep.name] = true;
             dep.dependencies.forEach(traverse);
             addArtifactFilePaths(dep, staticLibraryArtifacts);
             addExternalLibs(dep);
         } else if (dynamicLibraryArtifacts) {
+            seen[dep.name] = true;
             addArtifactFilePaths(dep, dynamicLibraryArtifacts);
         }
     }
@@ -304,7 +397,7 @@ function collectLibraryDependencies(product) {
 
 function linkerSupportsWholeArchive(product)
 {
-    return Utilities.versionCompare(product.cpp.compilerVersion, "19.0.25123") >= 0
+    return Utilities.versionCompare(product.cpp.compilerVersion, "19.0.24215.1") >= 0
 }
 
 function handleDiscardProperty(product, flags) {
@@ -324,45 +417,76 @@ function prepareLinker(project, product, inputs, outputs, input, output) {
         function (a) {
             return a.filePath;
         });
+    var moduleDefinitionInputs = Array.prototype.map.call(inputs["def"],
+        function (a) {
+            return a.filePath;
+        });
     var generateManifestFiles = !linkDLL && product.cpp.generateManifestFile;
-    var canEmbedManifest = (product.cpp.compilerVersionMajor >= 17);    // VS 2012
+    var useClangCl = product.qbs.toolchain.contains("clang-cl");
+    var canEmbedManifest = useClangCl || product.cpp.compilerVersionMajor >= 17 // VS 2012
 
-    var args = ['/nologo']
+    var linkerPath = effectiveLinkerPath(product, inputs);
+    var useCompilerDriver = linkerPath === product.cpp.compilerPath;
+    // args variable is built as follows:
+    // [linkerWrapper] linkerPath /nologo [driverFlags driverLinkerFlags]
+    //      allInputs libDeps [/link] linkerArgs
+    var args = []
+
+    if (useCompilerDriver) {
+        args.push('/nologo');
+        var driverFlags = product.cpp.driverFlags;
+        if (driverFlags)
+            args = args.concat(driverFlags);
+        var driverLinkerFlags = product.cpp.driverLinkerFlags;
+        if (driverLinkerFlags)
+            args = args.concat(driverLinkerFlags);
+    }
+
+    var allInputs = inputs.obj || [];
+    for (i in allInputs) {
+        var fileName = FileInfo.toWindowsSeparators(allInputs[i].filePath)
+        args.push(fileName)
+    }
+
+    var linkerArgs = ['/nologo']
     if (linkDLL) {
-        args.push('/DLL');
-        args.push('/IMPLIB:' + FileInfo.toWindowsSeparators(outputs.dynamiclibrary_import[0].filePath));
+        linkerArgs.push('/DLL');
+        linkerArgs.push('/IMPLIB:' + FileInfo.toWindowsSeparators(outputs.dynamiclibrary_import[0].filePath));
     }
 
     if (debugInformation) {
-        args.push("/DEBUG");
+        linkerArgs.push("/DEBUG");
         var debugInfo = outputs.debuginfo_app || outputs.debuginfo_dll;
         if (debugInfo)
-            args.push("/PDB:" + debugInfo[0].fileName);
+            linkerArgs.push("/PDB:" + debugInfo[0].fileName);
     } else {
-        args.push('/INCREMENTAL:NO')
+        linkerArgs.push('/INCREMENTAL:NO')
     }
 
     switch (product.qbs.architecture) {
     case "x86":
-        args.push("/MACHINE:X86");
+        linkerArgs.push("/MACHINE:X86");
         break;
     case "x86_64":
-        args.push("/MACHINE:X64");
+        linkerArgs.push("/MACHINE:X64");
         break;
     case "ia64":
-        args.push("/MACHINE:IA64");
+        linkerArgs.push("/MACHINE:IA64");
         break;
     case "armv7":
-        args.push("/MACHINE:ARM");
+        linkerArgs.push("/MACHINE:ARM");
         break;
     case "arm64":
-        args.push("/MACHINE:ARM64");
+        linkerArgs.push("/MACHINE:ARM64");
         break;
     }
 
+    if (useCompilerDriver)
+        handleClangClArchitectureFlags(product, product.qbs.architecture, args);
+
     var requireAppContainer = product.cpp.requireAppContainer;
     if (requireAppContainer !== undefined)
-        args.push("/APPCONTAINER" + (requireAppContainer ? "" : ":NO"));
+        linkerArgs.push("/APPCONTAINER" + (requireAppContainer ? "" : ":NO"));
 
     var minimumWindowsVersion = product.cpp.minimumWindowsVersion;
     var subsystemSwitch = undefined;
@@ -372,25 +496,29 @@ function prepareLinker(project, product, inputs, outputs, input, output) {
         subsystemSwitch = product.consoleApplication === false ? '/SUBSYSTEM:WINDOWS' : '/SUBSYSTEM:CONSOLE';
     }
 
+    var useLldLink = useCompilerDriver && product.cpp.linkerVariant === "lld"
+            || !useCompilerDriver && product.cpp.linkerName === "lld-link.exe";
     if (minimumWindowsVersion) {
         var subsystemVersion = WindowsUtils.getWindowsVersionInFormat(minimumWindowsVersion,
                                                                       'subsystem');
         if (subsystemVersion) {
             subsystemSwitch += ',' + subsystemVersion;
-            args.push('/OSVERSION:' + subsystemVersion);
+            // llvm linker does not support /OSVERSION
+            if (!useLldLink)
+                linkerArgs.push('/OSVERSION:' + subsystemVersion);
         }
     }
 
     if (subsystemSwitch)
-        args.push(subsystemSwitch);
+        linkerArgs.push(subsystemSwitch);
 
     var linkerOutputNativeFilePath = FileInfo.toWindowsSeparators(primaryOutput.filePath);
     var manifestFileNames = [];
     if (generateManifestFiles) {
         if (canEmbedManifest) {
-            args.push("/MANIFEST:embed");
+            linkerArgs.push("/MANIFEST:embed");
             additionalManifestInputs.forEach(function (manifestFileName) {
-                args.push("/MANIFESTINPUT:" + manifestFileName);
+                linkerArgs.push("/MANIFESTINPUT:" + manifestFileName);
             });
         } else {
             linkerOutputNativeFilePath
@@ -399,24 +527,36 @@ function prepareLinker(project, product, inputs, outputs, input, output) {
                             + primaryOutput.fileName);
 
             var manifestFileName = linkerOutputNativeFilePath + ".manifest";
-            args.push('/MANIFEST', '/MANIFESTFILE:' + manifestFileName);
+            linkerArgs.push('/MANIFEST', '/MANIFESTFILE:' + manifestFileName);
             manifestFileNames = [manifestFileName].concat(additionalManifestInputs);
         }
     }
 
-    var allInputs = inputs.obj || [];
-    for (i in allInputs) {
-        var fileName = FileInfo.toWindowsSeparators(allInputs[i].filePath)
-        args.push(fileName)
-    }
+    if (moduleDefinitionInputs.length === 1)
+        linkerArgs.push("/DEF:" + moduleDefinitionInputs[0]);
+    else if (moduleDefinitionInputs.length > 1)
+        throw new Error("Only one '.def' file can be specified for linking");
 
     var wholeArchiveSupported = linkerSupportsWholeArchive(product);
     var wholeArchiveRequested = false;
     var libDeps = collectLibraryDependencies(product);
+    var prevLib;
     for (i = 0; i < libDeps.length; ++i) {
         var dep = libDeps[i];
-        args.push((wholeArchiveSupported && dep.wholeArchive ? "/WHOLEARCHIVE:" : "")
-                  + FileInfo.toWindowsSeparators(dep.filePath));
+        var lib = dep.filePath;
+        if (lib === prevLib)
+            continue;
+        prevLib = lib;
+
+        if (wholeArchiveSupported && dep.wholeArchive) {
+            // need to pass libraries to the driver to avoid "no input files" error if no object
+            // files are specified; thus libraries are duplicated when using "WHOLEARCHIVE"
+            if (useCompilerDriver && allInputs.length === 0)
+                args.push(FileInfo.toWindowsSeparators(lib));
+            linkerArgs.push("/WHOLEARCHIVE:" + FileInfo.toWindowsSeparators(lib));
+        } else {
+            args.push(FileInfo.toWindowsSeparators(lib));
+        }
         if (dep.wholeArchive)
             wholeArchiveRequested = true;
     }
@@ -427,37 +567,71 @@ function prepareLinker(project, product, inputs, outputs, input, output) {
     }
 
     if (product.cpp.entryPoint)
-        args.push("/ENTRY:" + product.cpp.entryPoint);
+        linkerArgs.push("/ENTRY:" + product.cpp.entryPoint);
 
-    args.push('/OUT:' + linkerOutputNativeFilePath)
+    if (outputs.application && product.cpp.generateLinkerMapFile) {
+        if (useLldLink)
+            linkerArgs.push("/lldmap:" + outputs.mem_map[0].filePath);
+        else
+            linkerArgs.push("/MAP:" + outputs.mem_map[0].filePath);
+    }
+
+    if (useCompilerDriver)
+        args.push('/Fe' + linkerOutputNativeFilePath);
+    else
+        linkerArgs.push('/OUT:' + linkerOutputNativeFilePath);
     var libraryPaths = product.cpp.libraryPaths;
     if (libraryPaths)
         libraryPaths = [].uniqueConcat(libraryPaths);
     for (i in libraryPaths) {
-        args.push('/LIBPATH:' + FileInfo.toWindowsSeparators(libraryPaths[i]))
+        linkerArgs.push('/LIBPATH:' + FileInfo.toWindowsSeparators(libraryPaths[i]))
     }
-    handleDiscardProperty(product, args);
+    handleDiscardProperty(product, linkerArgs);
     var linkerFlags = product.cpp.platformLinkerFlags.concat(product.cpp.linkerFlags);
-    args = args.concat(linkerFlags);
+    linkerArgs = linkerArgs.concat(linkerFlags);
     if (product.cpp.allowUnresolvedSymbols)
-        args.push("/FORCE:UNRESOLVED");
+        linkerArgs.push("/FORCE:UNRESOLVED");
 
-    var linkerPath = product.cpp.linkerPath;
     var wrapperArgs = product.cpp.linkerWrapper;
     if (wrapperArgs && wrapperArgs.length > 0) {
-        args.unshift(linkerPath);
+        linkerArgs.unshift(linkerPath);
         linkerPath = wrapperArgs.shift();
-        args = wrapperArgs.concat(args);
+        linkerArgs = wrapperArgs.concat(linkerArgs);
     }
     var commands = [];
+    var warningCmd = new JavaScriptCommand();
+    warningCmd.silent = true;
+    warningCmd.libDeps = libDeps;
+    warningCmd.sourceCode = function() {
+        for (var i = 0; i < libDeps.length; ++i) {
+            if (!libDeps[i].productName || File.exists(libDeps[i].filePath))
+                continue;
+            console.warn("Import library '" + FileInfo.toNativeSeparators(libDeps[i].filePath)
+                         + "' does not exist. This typically happens when a DLL does not "
+                         + "export any symbols. Please make sure the '" + libDeps[i].productName
+                         + "' library exports symbols, or, if you do not intend to actually "
+                         + "link against it, specify \"cpp.link: false\" in the Depends item.");
+        }
+    };
+    commands.push(warningCmd);
+
+    if (linkerArgs.length !== 0)
+        args = args.concat(useCompilerDriver ? ['/link'] : []).concat(linkerArgs);
+
     var cmd = new Command(linkerPath, args)
     cmd.description = 'linking ' + primaryOutput.fileName;
     cmd.highlight = 'linker';
+    cmd.jobPool = "linker";
     cmd.relevantEnvironmentVariables = ["LINK", "_LINK_", "LIB", "TMP"];
     cmd.workingDirectory = FileInfo.path(primaryOutput.filePath)
     cmd.responseFileUsagePrefix = '@';
+    cmd.responseFileSeparator = useCompilerDriver ? ' ' : '\n';
     cmd.stdoutFilterFunction = function(output) {
-        return output.replace(/^ +Creating library.*\r\n$/, "");
+        res = output.replace(/^.*performing full link.*\s*/, "");
+        res = res.replace(/^ *Creating library.*\s*/, "");
+        res = res.replace(/^\s*Generating code\s*/, "");
+        res = res.replace(/^\s*Finished generating code\s*/, "");
+        return res;
     };
     commands.push(cmd);
 
@@ -484,3 +658,46 @@ function prepareLinker(project, product, inputs, outputs, input, output) {
     return commands;
 }
 
+function createRcCommand(binary, input, output, logo) {
+    var platformDefines = input.cpp.platformDefines;
+    var defines = input.cpp.defines;
+    var includePaths = input.cpp.includePaths;
+    var systemIncludePaths = input.cpp.systemIncludePaths;
+
+    var args = [];
+    if (logo === "can-suppress-logo")
+        args.push("/nologo");
+    for (i in platformDefines) {
+        args.push("/d");
+        args.push(platformDefines[i]);
+    }
+    for (i in defines) {
+        args.push("/d");
+        args.push(defines[i]);
+    }
+    for (i in includePaths) {
+        args.push("/I");
+        args.push(includePaths[i]);
+    }
+    for (i in systemIncludePaths) {
+        args.push("/I");
+        args.push(systemIncludePaths[i]);
+    }
+    args = args.concat(["/FO", output.filePath, input.filePath]);
+    var cmd = new Command(binary, args);
+    cmd.description = 'compiling ' + input.fileName;
+    cmd.highlight = 'compiler';
+    cmd.jobPool = "compiler";
+
+    if (logo === "always-shows-logo") {
+        // Remove the first two lines of stdout. That's the logo.
+        cmd.stdoutFilterFunction = function(output) {
+            var idx = 0;
+            for (var i = 0; i < 3; ++i)
+                idx = output.indexOf('\n', idx + 1);
+            return output.substr(idx + 1);
+        }
+    }
+
+    return cmd;
+}
