@@ -45,6 +45,7 @@
 #include <logging/translator.h>
 #include <tools/architectures.h>
 #include <tools/profile.h>
+#include <tools/qttools.h>
 #include <tools/settings.h>
 
 #include <QtCore/qstringlist.h>
@@ -59,16 +60,60 @@
 using namespace qbs;
 using Internal::Tr;
 
-namespace {
 static const QString defaultDeveloperPath =
         QStringLiteral("/Applications/Xcode.app/Contents/Developer");
 static const std::regex defaultDeveloperPathRegex(
-        "^/Applications/Xcode([a-zA-Z0-9 _-]+)\\.app/Contents/Developer$");
+        "^/Applications/Xcode([a-zA-Z0-9 ._-]+)\\.app/Contents/Developer$");
 
+static QString targetOS(const QString &applePlatformName)
+{
+    if (applePlatformName == QStringLiteral("macosx"))
+        return QStringLiteral("macos");
+    if (applePlatformName == QStringLiteral("iphoneos"))
+        return QStringLiteral("ios");
+    if (applePlatformName == QStringLiteral("iphonesimulator"))
+        return QStringLiteral("ios-simulator");
+    if (applePlatformName == QStringLiteral("appletvos"))
+        return QStringLiteral("tvos");
+    if (applePlatformName == QStringLiteral("appletvsimulator"))
+        return QStringLiteral("tvos-simulator");
+    if (applePlatformName == QStringLiteral("watchos"))
+        return QStringLiteral("watchos");
+    if (applePlatformName == QStringLiteral("watchsimulator"))
+        return QStringLiteral("watchos-simulator");
+    return {};
+}
+
+static QStringList archList(const QString &applePlatformName)
+{
+    QStringList archs;
+    if (applePlatformName == QStringLiteral("macosx")
+            || applePlatformName == QStringLiteral("iphonesimulator")
+            || applePlatformName == QStringLiteral("appletvsimulator")
+            || applePlatformName == QStringLiteral("watchsimulator")) {
+        if (applePlatformName != QStringLiteral("appletvsimulator"))
+            archs << QStringLiteral("x86");
+        if (applePlatformName != QStringLiteral("watchsimulator"))
+            archs << QStringLiteral("x86_64");
+        if (applePlatformName == QStringLiteral("macosx"))
+            archs << QStringLiteral("arm64");
+    } else if (applePlatformName == QStringLiteral("iphoneos")
+               || applePlatformName == QStringLiteral("appletvos")) {
+        if (applePlatformName != QStringLiteral("appletvos"))
+            archs << QStringLiteral("armv7a");
+        archs << QStringLiteral("arm64");
+    } else if (applePlatformName == QStringLiteral("watchos")) {
+        archs << QStringLiteral("armv7k");
+    }
+
+    return archs;
+}
+
+namespace {
 class XcodeProbe
 {
 public:
-    XcodeProbe(qbs::Settings *settings, QList<qbs::Profile> &profiles)
+    XcodeProbe(qbs::Settings *settings, std::vector<qbs::Profile> &profiles)
         : settings(settings), profiles(profiles)
     { }
 
@@ -78,7 +123,7 @@ public:
     void detectAll();
 private:
     qbs::Settings *settings;
-    QList<qbs::Profile> &profiles;
+    std::vector<qbs::Profile> &profiles;
     QStringList developerPaths;
 };
 
@@ -117,65 +162,20 @@ void XcodeProbe::detectDeveloperPaths()
     if (!launchServices.waitForFinished(-1) || launchServices.exitCode()) {
         qbsInfo() << Tr::tr("Could not detect additional Xcode installations with /usr/bin/mdfind");
     } else {
-        for (const QString &path : QString::fromLocal8Bit(launchServices.readAllStandardOutput())
-             .split(QLatin1Char('\n'), QString::SkipEmptyParts))
+        const auto paths = QString::fromLocal8Bit(launchServices.readAllStandardOutput())
+                .split(QLatin1Char('\n'), QBS_SKIP_EMPTY_PARTS);
+        for (const QString &path : paths)
             addDeveloperPath(path + QStringLiteral("/Contents/Developer"));
     }
 }
 
-static QString targetOS(const QString &applePlatformName)
-{
-    if (applePlatformName == QStringLiteral("macosx"))
-        return QStringLiteral("macos");
-    if (applePlatformName == QStringLiteral("iphoneos"))
-        return QStringLiteral("ios");
-    if (applePlatformName == QStringLiteral("iphonesimulator"))
-        return QStringLiteral("ios-simulator");
-    if (applePlatformName == QStringLiteral("appletvos"))
-        return QStringLiteral("tvos");
-    if (applePlatformName == QStringLiteral("appletvsimulator"))
-        return QStringLiteral("tvos-simulator");
-    if (applePlatformName == QStringLiteral("watchos"))
-        return QStringLiteral("watchos");
-    if (applePlatformName == QStringLiteral("watchsimulator"))
-        return QStringLiteral("watchos-simulator");
-    return {};
-}
-
-static QStringList archList(const QString &applePlatformName)
-{
-    QStringList archs;
-    if (applePlatformName == QStringLiteral("macosx")
-            || applePlatformName == QStringLiteral("iphonesimulator")
-            || applePlatformName == QStringLiteral("appletvsimulator")
-            || applePlatformName == QStringLiteral("watchsimulator")) {
-        if (applePlatformName != QStringLiteral("appletvsimulator"))
-            archs << QStringLiteral("x86");
-        if (applePlatformName != QStringLiteral("watchsimulator"))
-            archs << QStringLiteral("x86_64");
-    } else if (applePlatformName == QStringLiteral("iphoneos")
-               || applePlatformName == QStringLiteral("appletvos")) {
-        if (applePlatformName != QStringLiteral("appletvos"))
-            archs << QStringLiteral("armv7a");
-        archs << QStringLiteral("arm64");
-    } else if (applePlatformName == QStringLiteral("watchos")) {
-        archs << QStringLiteral("armv7k");
-    }
-
-    return archs;
-}
-
 void XcodeProbe::setupDefaultToolchains(const QString &devPath, const QString &xcodeName)
 {
     qbsInfo() << Tr::tr("Profile '%1' created for '%2'.").arg(xcodeName).arg(devPath);
 
     Profile installationProfile(xcodeName, settings);
     installationProfile.removeProfile();
-    installationProfile.setValue(QStringLiteral("qbs.toolchain"), QStringList()
-                                 << QStringLiteral("xcode")
-                                 << QStringLiteral("clang")
-                                 << QStringLiteral("llvm")
-                                 << QStringLiteral("gcc"));
+    installationProfile.setValue(QStringLiteral("qbs.toolchainType"), QStringLiteral("xcode"));
     if (devPath != defaultDeveloperPath)
         installationProfile.setValue(QStringLiteral("xcode.developerPath"), devPath);
     profiles.push_back(installationProfile);
@@ -188,14 +188,15 @@ void XcodeProbe::setupDefaultToolchains(const QString &devPath, const QString &x
               << QStringLiteral("appletvsimulator")
               << QStringLiteral("watchos")
               << QStringLiteral("watchsimulator");
-    for (const QString &platform : platforms) {
+    for (const QString &platform : qAsConst(platforms)) {
         Profile platformProfile(xcodeName + QLatin1Char('-') + platform, settings);
         platformProfile.removeProfile();
         platformProfile.setBaseProfile(installationProfile.name());
         platformProfile.setValue(QStringLiteral("qbs.targetPlatform"), targetOS(platform));
         profiles.push_back(platformProfile);
 
-        for (const QString &arch : archList(platform)) {
+        const auto architectures = archList(platform);
+        for (const QString &arch : architectures) {
             Profile archProfile(xcodeName + QLatin1Char('-') + platform + QLatin1Char('-') + arch,
                                 settings);
             archProfile.removeProfile();
@@ -211,14 +212,15 @@ void XcodeProbe::detectAll()
 {
     int i = 1;
     detectDeveloperPaths();
-    for (const QString &developerPath : developerPaths) {
+    for (const QString &developerPath : qAsConst(developerPaths)) {
         QString profileName = QStringLiteral("xcode");
         if (developerPath != defaultDeveloperPath) {
             const auto devPath = developerPath.toStdString();
             std::smatch match;
             if (std::regex_match(devPath, match, defaultDeveloperPathRegex))
-                profileName += QString::fromStdString(match[1]).toLower().replace(QLatin1Char(' '),
-                                                                                  QLatin1Char('-'));
+                profileName += QString::fromStdString(match[1]).toLower().
+                        replace(QLatin1Char(' '), QLatin1Char('-')).
+                        replace(QLatin1Char('.'), QLatin1Char('_'));
             else
                 profileName += QString::number(i++);
         }
@@ -227,7 +229,7 @@ void XcodeProbe::detectAll()
 }
 } // end anonymous namespace
 
-void xcodeProbe(qbs::Settings *settings, QList<qbs::Profile> &profiles)
+void xcodeProbe(qbs::Settings *settings, std::vector<qbs::Profile> &profiles)
 {
     XcodeProbe probe(settings, profiles);
     probe.detectAll();
