@@ -1,3 +1,5 @@
+#include <utility>
+
 /****************************************************************************
 **
 ** Copyright (C) 2016 The Qt Company Ltd.
@@ -76,15 +78,16 @@ namespace qbs {
 namespace Internal {
 
 RulesApplicator::RulesApplicator(
-        const ResolvedProductPtr &product,
+        ResolvedProductPtr product,
         const std::unordered_map<QString, const ResolvedProduct *> &productsByName,
         const std::unordered_map<QString, const ResolvedProject *> &projectsByName,
-        const Logger &logger)
-    : m_product(product)
+        Logger logger)
+    : m_product(std::move(product))
+    // m_productsByName and m_projectsByName are references, cannot move-construct
     , m_productsByName(productsByName)
     , m_projectsByName(projectsByName)
     , m_mocScanner(nullptr)
-    , m_logger(logger)
+    , m_logger(std::move(logger))
 {
 }
 
@@ -186,7 +189,7 @@ void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &p
     qCDebug(lcBuildGraph) << "apply rule" << m_rule->toString()
                           << toStringList(inputArtifacts).join(QLatin1String(",\n            "));
 
-    QList<std::pair<const RuleArtifact *, OutputArtifactInfo>> ruleArtifactArtifactMap;
+    std::vector<std::pair<const RuleArtifact *, OutputArtifactInfo>> ruleArtifactArtifactMap;
     QList<Artifact *> outputArtifacts;
 
     m_transformer = Transformer::create();
@@ -211,13 +214,13 @@ void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &p
                     ScriptEngine::argumentList(Rule::argumentNamesForOutputArtifacts(), scope()));
     } else {
         Set<QString> outputFilePaths;
-        for (const RuleArtifactConstPtr &ruleArtifact : m_rule->artifacts) {
+        for (const auto &ruleArtifact : m_rule->artifacts) {
             const OutputArtifactInfo outputInfo = createOutputArtifactFromRuleArtifact(
                         ruleArtifact, inputArtifacts, &outputFilePaths);
             if (!outputInfo.artifact)
                 continue;
             outputArtifacts.push_back(outputInfo.artifact);
-            ruleArtifactArtifactMap.push_back({ ruleArtifact.get(), outputInfo });
+            ruleArtifactArtifactMap.emplace_back(ruleArtifact.get(), outputInfo);
         }
         if (m_rule->artifacts.empty()) {
             outputArtifacts.push_back(createOutputArtifactFromRuleArtifact(
@@ -285,7 +288,7 @@ void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &p
             }
             const QVariant value = scriptValue.toVariant();
             setConfigProperty(artifactModulesCfg, binding.name, value);
-            outputArtifact->pureProperties.push_back(std::make_pair(binding.name, value));
+            outputArtifact->pureProperties.emplace_back(binding.name, value);
         }
         outputArtifact->properties->setValue(artifactModulesCfg);
         if (!outputInfo.newlyCreated && (outputArtifact->fileTags() != outputInfo.oldFileTags
@@ -308,7 +311,7 @@ void RulesApplicator::doApply(const ArtifactSet &inputArtifacts, QScriptValue &p
             || m_oldTransformer->commands != m_transformer->commands
             || commandsNeedRerun(m_transformer.get(), m_product.get(), m_productsByName,
                                  m_projectsByName)) {
-        for (Artifact * const output : outputArtifacts) {
+        for (Artifact * const output : qAsConst(outputArtifacts)) {
             output->clearTimestamp();
             m_invalidatedArtifacts += output;
         }
@@ -352,7 +355,7 @@ ArtifactSet RulesApplicator::collectAdditionalInputs(const FileTags &tags, const
         }
 
         if (inputsSources.testFlag(Dependencies)) {
-            for (const ResolvedProductConstPtr &depProduct : product->dependencies) {
+            for (const auto &depProduct : product->dependencies) {
                 for (Artifact * const ta : depProduct->targetArtifacts()) {
                     if (ta->fileTags().contains(fileTag)
                             && !ta->fileTags().intersects(rule->excludedInputs)) {
@@ -545,8 +548,8 @@ class ArtifactBindingsExtractor
 {
     struct Entry
     {
-        Entry(const QString &module, const QString &name, const QVariant &value)
-            : module(module), name(name), value(value)
+        Entry(QString module, QString name, QVariant value)
+            : module(std::move(module)), name(std::move(name)), value(std::move(value))
         {}
 
         QString module;
@@ -558,9 +561,9 @@ class ArtifactBindingsExtractor
     static Set<QString> getArtifactItemPropertyNames()
     {
         Set<QString> s;
-        for (const PropertyDeclaration &pd :
-                 BuiltinDeclarations::instance().declarationsForType(
-                     ItemType::Artifact).properties()) {
+        const auto properties = BuiltinDeclarations::instance().declarationsForType(
+                ItemType::Artifact).properties();
+        for (const PropertyDeclaration &pd : properties) {
             s.insert(pd.name());
         }
         s.insert(StringConstants::explicitlyDependsOnProperty());
@@ -605,7 +608,7 @@ public:
         for (const auto &e : m_propertyValues) {
             const QStringList key{e.module, e.name};
             setConfigProperty(artifactCfg, key, e.value);
-            outputArtifact->pureProperties.push_back(std::make_pair(key, e.value));
+            outputArtifact->pureProperties.emplace_back(key, e.value);
         }
         outputArtifact->properties->setValue(artifactCfg);
     }
