@@ -1,41 +1,49 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qbs.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms and
-** conditions see http://www.qt.io/terms-conditions. For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
+**
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
 #ifndef QBS_PROPERTYDECLARATION_H
 #define QBS_PROPERTYDECLARATION_H
 
-#include <QSharedDataPointer>
+#include <QtCore/qshareddata.h>
+#include <QtCore/qstring.h>
 
 QT_BEGIN_NAMESPACE
-class QScriptValue;
-class QString;
 class QStringList;
 QT_END_NAMESPACE
 
@@ -56,19 +64,21 @@ public:
         PathList,
         String,
         StringList,
-        Variant
+        Variant,
+        VariantList,
     };
 
     enum Flag
     {
         DefaultFlags = 0,
-        ListProperty = 0x1,
+        ReadOnlyFlag = 0x1,
         PropertyNotAvailableInConfig = 0x2     // Is this property part of a project, product or file configuration?
     };
     Q_DECLARE_FLAGS(Flags, Flag)
 
     PropertyDeclaration();
-    PropertyDeclaration(const QString &name, Type type, Flags flags = DefaultFlags);
+    PropertyDeclaration(const QString &name, Type type, const QString &initialValue = QString(),
+                        Flags flags = DefaultFlags);
     PropertyDeclaration(const PropertyDeclaration &other);
     ~PropertyDeclaration();
 
@@ -78,6 +88,8 @@ public:
     bool isScalar() const;
 
     static Type propertyTypeFromString(const QString &typeName);
+    QString typeString() const;
+    static QString typeString(Type t);
 
     const QString &name() const;
     void setName(const QString &name);
@@ -88,11 +100,11 @@ public:
     Flags flags() const;
     void setFlags(Flags f);
 
-    const QScriptValue &allowedValues() const;
-    void setAllowedValues(const QScriptValue &v);
+    const QStringList &allowedValues() const;
+    void setAllowedValues(const QStringList &v);
 
     const QString &description() const;
-    void setDescripton(const QString &str);
+    void setDescription(const QString &str);
 
     const QString &initialValueSource() const;
     void setInitialValueSource(const QString &str);
@@ -101,6 +113,7 @@ public:
     void setFunctionArgumentNames(const QStringList &lst);
 
     bool isDeprecated() const;
+    bool isExpired() const;
     const DeprecationInfo &deprecationInfo() const;
     void setDeprecationInfo(const DeprecationInfo &deprecationInfo);
 
