@@ -1,30 +1,39 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qbs.
 **
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms and
-** conditions see http://www.qt.io/terms-conditions. For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file.  Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, The Qt Company gives you certain additional
-** rights.  These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
+**
+** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
@@ -32,8 +41,8 @@
 
 #if defined(Q_OS_WIN)
 #   define PSAPI_VERSION 1      // To use GetModuleFileNameEx from Psapi.lib on all Win versions.
-#   include <qt_windows.h>
-#   include <Psapi.h>
+#   include <QtCore/qt_windows.h>
+#   include <psapi.h>
 #elif defined(Q_OS_DARWIN)
 #   include <libproc.h>
 #elif defined(Q_OS_LINUX)
@@ -41,13 +50,13 @@
 #   include <unistd.h>
 #   include <cstdio>
 #elif defined(Q_OS_BSD4)
-# include <QFile>
-# if !defined(Q_OS_NETBSD)
-#   include <sys/user.h>
-# endif
+# include <QtCore/qfile.h>
 #   include <sys/cdefs.h>
 #   include <sys/param.h>
 #   include <sys/sysctl.h>
+# if !defined(Q_OS_NETBSD)
+#   include <sys/user.h>
+# endif
 #else
 #   error Missing implementation of processNameByPid for this platform.
 #endif
@@ -60,12 +69,12 @@ QString processNameByPid(qint64 pid)
 #if defined(Q_OS_WIN)
     HANDLE hProcess = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, DWORD(pid));
     if (!hProcess)
-        return QString();
-    wchar_t buf[MAX_PATH];
+        return {};
+    wchar_t buf[UNICODE_STRING_MAX_CHARS];
     const DWORD length = GetModuleFileNameEx(hProcess, NULL, buf, sizeof(buf) / sizeof(wchar_t));
     CloseHandle(hProcess);
     if (!length)
-        return QString();
+        return {};
     QString name = QString::fromWCharArray(buf, length);
     int i = name.lastIndexOf(QLatin1Char('\\'));
     if (i >= 0)
@@ -83,7 +92,8 @@ QString processNameByPid(qint64 pid)
     char buf[PATH_MAX];
     memset(buf, 0, sizeof(buf));
     sprintf(exePath, "/proc/%lld/exe", pid);
-    readlink(exePath, buf, sizeof(buf));
+    if (readlink(exePath, buf, sizeof(buf)) < 0)
+        return {};
     return FileInfo::fileName(QString::fromUtf8(buf));
 #elif defined(Q_OS_BSD4)
 # if defined(Q_OS_NETBSD)
@@ -100,22 +110,22 @@ QString processNameByPid(qint64 pid)
     u_int mib_len = sizeof(mib)/sizeof(u_int);
 
     if (sysctl(mib, mib_len, &kp, &len, NULL, 0) < 0)
-        return QString();
+        return {};
 
 # if defined(Q_OS_OPENBSD) || defined(Q_OS_NETBSD)
     if (kp.p_pid != pid)
-        return QString();
+        return {};
     QString name = QFile::decodeName(kp.p_comm);
 # else
     if (kp.ki_pid != pid)
-        return QString();
+        return {};
     QString name = QFile::decodeName(kp.ki_comm);
 # endif
     return name;
 
 #else
     Q_UNUSED(pid);
-    return QString();
+    return {};
 #endif
 }
 
