@@ -125,7 +125,7 @@ public:
 SettingsModel::SettingsModel(const QString &settingsDir, Settings::Scope scope, QObject *parent)
     : QAbstractItemModel(parent), d(new SettingsModelPrivate)
 {
-    d->settings.reset(new qbs::Settings(settingsDir));
+    d->settings = std::make_unique<qbs::Settings>(settingsDir);
     d->settings->setScopeForWriting(scope);
     d->readSettings();
 }
@@ -155,7 +155,7 @@ void SettingsModel::updateSettingsDir(const QString &settingsDir)
 {
     const Settings::Scope scope = d->scope();
     beginResetModel();
-    d->settings.reset(new qbs::Settings(settingsDir));
+    d->settings = std::make_unique<qbs::Settings>(settingsDir);
     d->settings->setScopeForWriting(scope);
     d->readSettings();
     endResetModel();
@@ -327,11 +327,12 @@ void SettingsModel::SettingsModelPrivate::readSettings()
 {
     qDeleteAll(rootNode.children);
     rootNode.children.clear();
-    for (const QString &topLevelKey : settings->directChildren(QString(), scope()))
+    const auto topLevelKeys = settings->directChildren(QString(), scope());
+    for (const QString &topLevelKey : topLevelKeys)
         addNodeFromSettings(&rootNode, topLevelKey);
     for (QVariantMap::ConstIterator it = additionalProperties.constBegin();
          it != additionalProperties.constEnd(); ++it) {
-        const QStringList nameAsList = it.key().split(QLatin1Char('.'), QString::SkipEmptyParts);
+        const QStringList nameAsList = it.key().split(QLatin1Char('.'), QBS_SKIP_EMPTY_PARTS);
         addNode(&rootNode, nameAsList.front(), nameAsList.mid(1), it.value());
     }
     dirty = false;
@@ -353,7 +354,8 @@ void SettingsModel::SettingsModelPrivate::addNodeFromSettings(Node *parentNode,
             = fullyQualifiedName.mid(fullyQualifiedName.lastIndexOf(QLatin1Char('.')) + 1);
     Node * const node = createNode(parentNode, nodeName);
     node->value = settingsValueToRepresentation(settings->value(fullyQualifiedName, scope()));
-    for (const QString &childKey : settings->directChildren(fullyQualifiedName, scope()))
+    const auto childKeys = settings->directChildren(fullyQualifiedName, scope());
+    for (const QString &childKey : childKeys)
         addNodeFromSettings(node, fullyQualifiedName + QLatin1Char('.') + childKey);
     dirty = true;
 }
